mata

/**************************************************************************/
/*                                                                        */
/*                                                                        */
/*   bvpmvp(...): mata function to estimate a multivariate probit model   */
/*                via a chain of bivariate probit models                  */
/*                                                                        */
/*                                                                        */
/* bvpmvp(ynames,xnames,ifc,quivar,rndvar,printres) takes six arguments:  */
/*                                                                        */
/*     1. ynames: a 1x1 string containing the names of the M>=2           */
/*          binary outcomes, e.g. yn ="y1 y2 y3 y4"                       */
/*                                                                        */
/*     2. xnames: a 1x1 string containing the names of the (K-1)          */
/*          covariates (a constant is added automatically),               */
/*          e.g. xn="age educ sex region price"                           */
/*                                                                        */
/*     3. ifc: a 1x1 string containing any "if" conditions for subset     */
/*          estimation, e.g. ic="if female==1 & income>10000"             */
/*          Set equal to "" for no subsetting.                            */
/*                                                                        */
/*     4. quivar: Use quivar=1 if display of the results of the bivariate */
/*          probit and suest estimates is to be suppressed, =0 else       */
/*                                                                        */
/*     5. rndvar: Specify rounding of displayed results,                  */
/*          e.g. rndvar=.0001                                             */
/*                                                                        */
/*     6. printres: Specify whether (=1) or not (=0) the estimates of the */
/*          averaged beta-hats and correlation matrix are displayed       */
/*          If printres=1, the output will also display "cut & paste"     */
/*          versions of the averaged beta-hat and estimated correlation   */
/*          matrixes.  These may be useful for input into subsequent      */
/*          Mata computations.                                            */
/*                                                                        */
/*                                                                        */
/*     Examples:   bv1=bvpmvp(yn,xn,ic,0,.0001,1)                         */
/*                 bv2=bvpmvp("y1 y2 y3","x1 x2 x3","",1,.001,0)          */
/*                                                                        */
/*                                                                        */
/*  bvpmvp(...) returns the px(p+1) matrix, p=M*(K+.5*(M-1)), whose       */
/*   first column is the stacked vector containing the averaged beta-hats */
/*   atop the vech of the atanh of the estimated correlation matrix       */
/*   (excluding the diagonal elements), i.e.:                             */
/*                                                                        */
/*         t=[bA1',...,bAM',athrho(1,2),...,athrho((M-1),M)]',            */
/*                                                                        */
/*   and whose remaining pxp matrix is the estimated vcov(t).             */
/*                                                                        */
/*                                                                        */
/*   Note: This function uses Stata's suest to combine estimates.         */
/*         Since suest can accommodate combinations of estimates that are */
/*         based on different estimation samples, missingness of          */
/*         data on outcomes (and covariates) across observations is       */
/*         permissible.                                                   */
/*                                                                        */
/*   Note: Executing this mata function clears any stored estimates.      */
/*                                                                        */
/*   Note: For large problems it may be necessary to adjust Stata's       */
/*         matsize using "set matsize..."  This is so suest does not      */
/*         encounter size limitations when computing the combined         */
/*         parameter vector and its vcov matrix.  It is possible that     */
/*         some very large problems (i.e. M*K very large) cannot be       */
/*         handled due to Stata's internal constraint of matsize<=11,000. */
/*                                                                        */
/**************************************************************************/

function bvpmvp(ynames,xnames,ifc,quivar,rndvar,printres) {

/****** Declarations and Initializations ***/

real ba,va,bamat,samat,r,m,k,wt,rnd,sub,suv,rb,rind,rsel,rall,tall,vall,rtmp,sn,stmp
string xn,xn1,yn,xt,yt,sp,en,cn,qv,rmat,rp,lp,sun,ytmp,xtmp,blmat,icv,rs,rdis,nlcs,cm,cpmb,cpmr

cn="_cons"
xn1=xnames
sp=" "
lp="("
rp=")"
cm=","
xn=xnames+sp+cn
yn=ynames
en="ests"
if (quivar==0) qv=""
else qv="qui "
xt=tokens(xn)
yt=tokens(yn)
m=cols(yt)
k=cols(xt)
r=I(m)
rs=J(m,m," ")
wt=(1/(m-1))
rnd=rndvar
icv=ifc
nlcs="exp(2*_b[athrho:_cons])"

/****** Estimate Bivariate Probits; Store Results; Compute R Matrix ***/

stata("estimates clear")
  
for (i=1; i<=(m-1); i++) {
  for (j=(i+1); j<=m; j++)   {
    stata(qv+"biprobit "+yt[1,i]+sp+yt[1,j]+sp+xn1+sp+icv)
    stata(qv+"estimates store "+en+strofreal(i)+strofreal(j))
    stata("qui nlcom ("+nlcs+"-1)/("+nlcs+"+1), post")
    rtmp=st_matrix("e(b)")
    stmp=sqrt(st_matrix("e(V)"))
    r[i,j]=rtmp
    r[j,i]=r[i,j]
    rs[i,j]=lp+strofreal(round(stmp,rnd))+rp
    rs[j,i]=rs[i,j]
  }
} 

/****** Combine Estimates Using suest; Retrieve Combined Estimates ***/

stata(qv+"suest "+en+"*")
sun=st_matrixcolstripe("e(b)")
sub=st_matrix("e(b)")
suv=st_matrix("e(V)")
sn=st_numscalar("e(N)")
  
/****** Compute Averages of BVP Beta-Hat Point Estimates ***/

rb=J(0,(m*(m-1)*(k+.5)),.)
for (i=1; i<=m; i++) {
  ytmp=yt[.,i]
  for (j=1; j<=k; j++) {
    xtmp=xt[.,j]
    rb=rb\((substr(sun[.,1],-strlen(ytmp)):==ytmp):*(sun[.,2]:==xtmp))'
  }
}

rb=wt*rb
ba=rb*sub'
va=rb*suv*rb'
bamat=rowshape(ba,m)'
samat=rowshape(sqrt(diagonal(va)),m)'

/****** Construct Selection Matrix for athrho Estimates ***/ 

rind=selectindex(strpos(sun[.,1],"athrho"):>0)
rsel=J((.5*m*(m-1)),rows(sun),0)
for (i=1; i<=(.5*m*(m-1)); i++) {
  rsel[i,rind[i,.]]=1
}

/****** Compute Full Parameter Estimate Vector & Associated vcov Matrix ***/

rall=rb\rsel
tall=rall*sub'
vall=rall*suv*rall'

/****** Display Results ***/

if (printres==1) {
  blmat=J(1,(m+1)," ")
  rmat=(" ",yt)\blmat

  for (i=1; i<=k; i++) {
    rmat=rmat\(xt[.,i],strofreal(round(bamat[i,.],rnd)))
    rmat=rmat\(sp,(lp:+strofreal(round(samat[i,.],rnd)):+rp))
    rmat=rmat\blmat
  }
  " "
  "**********************************************"
  "*                                            *"
  "*       Multivariate Probit: Results         *"
  "*                                            *"
  "**********************************************"
  " "
  "N. of Observations (from suest): "+strofreal(sn)
  " "
  if (icv != "") {
    "Estimation Sample: "+icv
  }
  " "
  " "
  "Averaged Beta-Hat Point Estimates and Estimated Standard Errors"
  " "
  rmat
  " "
  " "
  "Estimated Correlation (Rho) Matrix and Estimated Standard Errors"
  " "
  rdis=(" ",yt)\J(1,(m+1)," ")
  for (i=1; i<=m; i++) {
    rdis=rdis\(yt[.,i],strofreal(round(r[i,.],rnd)))
    rdis=rdis\(" ",rs[i,.])
    rdis=rdis\J(1,(m+1)," ")
  }
  rdis

  cpmb=sp:+sp:+lp:+strofreal(round(bamat[.,1],rnd))
  for (i=2;i<=cols(bamat);i++) {
    cpmb=cpmb:+sp:+cm:+sp:+strofreal(round(bamat[.,i],rnd))
  }
  cpmb=cpmb:+rp:+sp:+(J(rows(bamat)-1,1,"\")\sp)

  " "
  " "
  "Cut & Paste Matrix, Averaged Beta-Hat Point Estimates"
  " "
  display(cpmb)
  " "
  " "
  cpmr=sp:+sp:+lp:+strofreal(round(r[.,1],rnd))
  for (i=2;i<=cols(r);i++) {
    cpmr=cpmr:+sp:+cm:+sp:+strofreal(round(r[.,i],rnd))
  }
  cpmr=cpmr:+rp:+sp:+(J(rows(r)-1,1,"\")\sp)

  "Cut & Paste Matrix, Estimated Correlation Matrix"
  " "
  display(cpmr)
  " "
  " "
}

stata("estimates clear")

return((tall,vall))
}


end

